import mariadb
import json
from pathlib import Path
from time import perf_counter
from pprint import pprint

conn = mariadb.connect(
       user="root",
       password="root",
       host="localhost",
       port=3306,
    )

c = conn.cursor()
c.execute('USE imdb')

with open('enums.json', encoding='utf-8') as enums_f:
    enums = json.load(enums_f)
for key in enums:  # transform to lookup table for ids: enums['genre']['Comedy'] => genre.id
    enums[key] = {name: i for i, name in enumerate(enums[key], 1)}


def idstrip(idstr, prefix_len=2):
    return int(idstr.strip()[prefix_len:])

def id_helper(parts, column, col_name, prefix_len):
    return idstrip(parts[column[col_name]], prefix_len)

def value_helper(parts, column, col_name, cls):
    cv = parts[column[col_name]].strip()
    return cls(cv) if cv != r'\N' else None

def enum_helper(parts, column, col_name, enum_name):
    v = value_helper(parts, column, col_name, str)
    return enums[enum_name][v] if v is not None else None

# valhelp and idhelp uses the parts and column directly from current scope
valhelp = lambda col_name, cls=str: value_helper(parts, column, col_name, cls)
idhelp = lambda  col_name, prefix_len=2: id_helper(parts, column, col_name, prefix_len)
enmhelp = lambda col_name, enum_name: enum_helper(parts, column, col_name, enum_name)

#c.execute('TRUNCATE TABLE title_crew')  # (only for repeated cell running in notebook)
t1 = perf_counter()
with open(Path('data') / 'title.crew.tsv', encoding='utf-8') as f:
    column = {}
    for i, p in enumerate(f.readline().strip().split('\t')):
        column[p.strip()] = i
    for line in f:
        parts = line.strip().split('\t')
        t_id = idhelp('tconst')  # ommit prefix
        glistD = parts[column['directors']].strip().split(',')
        glistW = parts[column['writers']].strip().split(',')
        if len(glistD) and glistD[0] == r'\N':
            glistD.clear()
        if len(glistW) and glistW[0] == r'\N':
            glistW.clear()
        D = set(idstrip(p) for p in glistD)
        W = set(idstrip(p) for p in glistW)

        for person_id in D.union(W):
            role = 'director_writer' if person_id in D and person_id in W else ('director' if person_id in D else 'writer')
            c.execute('INSERT INTO title_crew (title_id, person_id, role) VALUES (?,?,?)', (t_id, person_id, role))
print(perf_counter() - t1, 's')
